<?php
/**
 * A collection of classes extended from WebMATDatabaseTable
 * 
 * You need an extended class for each table in you module not including the 
 * dropdown tables.
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Calendar
 */

/**
 * This class handles the database functions for the item table
 * 
 * This module is an extend verison of WebMATDatabaseTable with the table_name 
 * set and the alert_where set.
 * 
 * @package Calendar
 */
class itemCalendarDatabaseTable extends WebMATDatabaseTable {
	
	/**
	 * @var $access_cache
	 * @access private
	 */
	var $access_cache = array();
	
	/**
	 * @var $access_cache
	 * @access private
	 */
	var $creator_cache = array();
	
	/**
	 * Role key
	 * @var $role_attendee
	 * @access private
	 */
	var $role_no_need_to_attend = 'NoNeedToAttend';
	
	/**
	 * The constructor class for itemCalendarDatabaseTable
	 *
	 * @param string $module_name the module name
	 * @param array $data_definition the data definition
	 */
	function itemCalendarDatabaseTable($module_name, $data_definition) {
		global $USER;
		$this->table_name = 'item';
		$this->table_name_user = 'item_user';
		$this->join_tables = $module_name . '__' . $this->table_name . ' LEFT JOIN ' . $module_name . '__' . $this->table_name_user . ' ON ' .
			$module_name . '__' . $this->table_name . '.id = ' . $module_name . '__' . $this->table_name_user . '.item_id
			LEFT JOIN Contact__item as _contact_id ON ' . $module_name . '__' . $this->table_name . '.contact_id = _contact_id.id';
		$this->WebMATDatabaseTable($module_name, $data_definition); //calls the constructor for WebMATDatabaseTable
	}
	
	/**
	 * Queries the Calendar access table. Results are cached.
	 * 
	 * @param mixed Either integer user_id, or array of user ids
	 * @param integer user_id The user requesting access
	 * @return boolean The access level the requesting user has for items created/assigned to specified user(s) 
	 * @author Phil Calder
	 */
	function getAccess($target_users, $requesting_user) {
		if (!is_array($target_users)) {
			$target_users = (array)$target_users;
		}
		$target_count = count($target_users);
		$cache_count = 0;
		$max_access = 0;
		foreach ($target_users as $k => $creator_id) {
			if ($creator_id && $requesting_user && isset($this->access_cache) && is_array($this->access_cache) ) {
				if (array_key_exists($creator_id, $this->access_cache) ) {
					if (array_key_exists($requesting_user, $this->access_cache[$creator_id]) ) {
						$cache_count++;
						$max_access = max($max_access, $this->access_cache[$creator_id][$requesting_user]);
						if ( ($max_access == 3) || ($cache_count == $target_count)) { // 3 is full access
							// print '<div>returning cached max access: ' . $max_access . '</div>';
							return $max_access;
						}
					}
				}
			}
		}
		
		$creator_id_string = implode(',', $target_users);
		$access_for_target_users = $this->db->quickGetAllRows(
			'creator_id,access', 
			'Calendar__access', 
			'creator_id IN (' . $creator_id_string . ') AND user_id=' . $requesting_user
		);
		if (is_array($access_for_target_users)) {
			foreach ($access_for_target_users as $k => $row) {
				$creator_id = $row['creator_id'];
				$access = $row['access'];
				$max_access = max($max_access, $access);
				$this->access_cache[$creator_id][$requesting_user] = $access;
			}
		}
		// print '<div>returning max access: ' . $max_access . '</div>';
		return $max_access;
	}
	
	/**
	 * Queries the Calendar item table. Results are cached.
	 * 
	 * @param int $id The Calendar item id
	 * @return int The Calendar item creator_id
	 * @author Phil Calder
	 */
	function getCreatorId($id) {
		if ($id && is_array($this->creator_cache) && isset($this->creator_cache[$id]) && $this->creator_cache[$id]) {
			return $this->creator_cache[$id];
		}
		$this->creator_cache[$id] = $this->db->quickGet('creator_id', $this->table_name, "id=$id");
		return $this->creator_cache[$id];
	}
	
	/**
	 * The current user can view the given item if s/he has at least view rights 
	 * for the creator of this item, or a user this item is assigned to
	 *
	 * Secondary condition: user may need to have selected to view Calendar items for a particular user
	 * 
	 * @return boolean Whether the given item is viewable by the current user
	 * @author Phil Calder
	 */
	function isViewable($id, $item = false) {
		global $USER;
		$id = $this->cleanId($id); // Phil 20070305
		// print '<div>isViewable(' . $id . ')</div>'; print_r($item); print '<hr>';
		
		// Phil 20060524 Get cached value
		// Can the current user view items created by the creator of this item
		if (!isset($item['creator_id']) || !$item['creator_id']) {
			$creator_id = $this->getCreatorId($id);
		} else {
			$creator_id = $item['creator_id'];
		}
		if ($creator_id) {
			// Phil 20060524 Get cached value
			$access = $this->getAccess($creator_id, $USER['id']);
			if ($access) {
				return true;
			}
		} else {
			return false;
		}
		
		// Phil 20061013 Check access rights to all users of the item
		// Cant access through rights to view creators items. Try item users.
		$user_ids = $this->db->quickGetAllRows('user_id', 'Calendar__' . $this->table_name_user, "item_id='$id'");
		if (is_array($user_ids)) {
			// Phil 20060524 Get cached value
			$access = $this->getAccess($user_ids, $USER['id']);	
			if ($access) {
				return true;
			}
		}
		
		// TODO: optimise this
		$tables = $fields = '';
		$contact_fields = $this->processGetItemsContactFields($tables, $fields); // Alters the where statement for contact fields
		$foreign_fields = $this->processGetItemsForeignFields($tables, $fields); // Alters the where statement for foreign fields
		$this->processGetItemsItemUserTable($tables, $fields);
		$this->processJoinGetItemUserTable($tables); // Join item_user table to tables;
		$where = $this->getWhereStatement() . ' AND ' . $this->table_name . ".id=$id";
		
		$result = $this->db->quickGet('DISTINCT ' . $this->table_name . '.creator_id', $tables, $where);
		if ($result) {
			// print '<div>returning true after isViewable query for item ' . $id . '</div>';
			return true;
		}
		return false;
	}
	
	/**
	 * Whether the current user can edit the given item
	 *
	 * Conditions:
	 * item exits and is not cancelled
	 * Current user is the item creator
	 * Current user has access rights to edit items by the creator (and has selected to view items by that creator)
	 */
	function isEditable($id,  $item = false) {
		$id = $this->cleanId($id); // Phil 20070305
		// Phil 20061013 cached
		if (
			isset($this->cache[$this->module_name][$this->getTableName()][$id]['editable']) 
			&& $this->cache[$this->module_name][$this->getTableName()][$id]['editable']
		) {
			// print '<div>isEditable(' . $id . ') cached</div>';
			return $this->cache[$this->module_name][$this->getTableName()][$id]['editable'];
		}
		
		global $USER;
		// print '<div>isEditable(' . $id . ')</div>';
		if (!$id) {
			return ($this->cache[$this->module_name][$this->getTableName()][$id]['editable'] = false);
		}
		if (!$item) {
			$item = $this->db->quickGet('*', $this->table_name, "id=$id AND is_cancelled=0");
			if (!$item) {
				return ($this->cache[$this->module_name][$this->getTableName()][$id]['editable'] = false);
				
			}
		}
		if ($item['is_cancelled']) {
			return ($this->cache[$this->module_name][$this->getTableName()][$id]['editable'] = false);
		}
		if ($item['creator_id'] == $USER['id']) {
			return ($this->cache[$this->module_name][$this->getTableName()][$id]['editable'] = true);
		}
		// Phil 20060524 Get cached value (access to items by this items creator)
		$access = $this->getAccess($item['creator_id'], $USER['id']);	
		if ($access >= 3) {
			// print '<div>isEditable returning true (user ' . $USER['id'] . ' can access creator ' . $item['creator_id'] . ')</div>';
			return ($this->cache[$this->module_name][$this->getTableName()][$id]['editable'] = true);
		}
		return ($this->cache[$this->module_name][$this->getTableName()][$id]['editable'] = false);
	}
	
	function countAlerts() {
		if ($alert_ids = $this->getAlertIds()) {
			return count($alert_ids);
		}
		return 0;
	}
	
	function countItems() {  //count all
		global $SERVICES, $USER;
		$tables = $fields = '';
		$contact_fields = $this->processGetItemsContactFields($tables, $fields); // Alters the where statement for contact fields
		$foreign_fields = $this->processGetItemsForeignFields($tables, $fields); // Alters the where statement for foreign fields
		$this->processGetItemsItemUserTable($tables, $fields);
		$this->processJoinGetItemUserTable($tables); // Join item_user table to tables;
		$where = $this->getWhereStatement();
		return $this->db->quickGet("count(DISTINCT Calendar__item.id)", $tables, $where);
	}
	
	function deleteItem($id) {
		global $SERVICES, $L;
		$this->db->update($this->getTableName(), array('is_cancelled' => '1'), "id='$id'"); 
		if ($this->db->hasError()) {
			$this->db->printError();
		} else {
			$this->setLog($L['logDelete'], 'id=' . $id);
		}
	}
	
	// Join item_user table to tables;
	function processJoinGetItemUserTable(&$tables) {
		$tables .= ' LEFT JOIN ' . $this->module_name . '__' . $this->table_name_user . ' ON '
			 				. $this->table_name . '.id = ' . $this->module_name . '__' . $this->table_name_user . '.item_id';
	}

	/**
	 * Overridden to insert seen table rows for users
	 * @author Bosco 20051129
	 */	
	function setItem($item) {
		$user_ids = $item['user_id'];
		bcrm_unset($item, 'user_id');
		$id = parent::setItem($item);
		if ($id) {
			$this->setUserSeenItem($id, $user_ids);	
		}
		return $id;
	}
	
	/**
	 * Add the items to the item_seen table
	 *
	 * @param integer $item_id the current item id
	 * @param array $user_ids an array or user id's
	 */
	function setUserSeenItem($item_id, $user_ids) {
		global $SERVICES, $USER;
		$groups =& $SERVICES->getService('Groups');
		$user_ids = $groups->getUsers($user_ids);
		if (is_array($user_ids) && sizeof($user_ids)) {
			// insert into seen: item_id, user_id, seen=0
			bcrm_unset($user_ids, $USER['id']);
			$mod =& $SERVICES->modules->getModule($this->module_name);
			$db_functions =& $mod->getDatabaseTable('item_seen');
			foreach ($user_ids as $id => $user) {
				$item_seen = array (
						'item_id' => $item_id,
						'user_id' => $id,
						'seen' => 0
				);
				$db_functions->setItem($item_seen);
			}
		}
	}

	function getItems($order_by = false, $limit = false, $where = false) {  
		global $SERVICES, $USER;
		$sort = $this->getOrderBy($order_by);
		$contact_fields = array();
		$tables = $fields = '';	// set in processGetItemsContactFields and processGetItemsForeignFields
		$contact_fields = $this->processGetItemsContactFields($tables, $fields);	
		$foreign_fields = $this->processGetItemsForeignFields($tables, $fields);			
		$this->processGetItemsItemUserTable($tables, $fields);
		$this->processJoinGetItemUserTable($tables); // Join item_user table to tables;
		if ($where) {
			$where = $where;
		} else {
			$where = $this->getWhereStatement();
		}
		
		// Phil 20061013 Necessary to add DISTINCT here because where statement would otherwise return 
		// multiple rows (from join with item user table) per item id
		$fields = 'DISTINCT (Calendar__item.id) as c_id, ' . $fields;
		
		// print '<div><strong>SELECT</strong> ' . $fields . '</div>';
		// print '<div><strong>FROM</strong> ' . $tables . '</div>';
		// print '<div><strong>WHERE</strong> ' . $where . '</div>';
		// if ($sort) { print '<div><strong>ORDER BY</strong> ' . $sort . '</div>'; }
		// if ($limit) { print '<div>' . $limit . '</div>'; }
		
		// $result = $this->db->select("DISTINCT $fields", $tables, $where , $sort , '' ,$limit );
		$result = $this->db->select($fields, $tables, $where , $sort , '' ,$limit );
		if ($this->db->hasError())	$this->db->printError();
		while ($row = $this->db->getRow($result)) {
			$items[$row['id']] = $this->processRow($row['id'], $row); // Phil 20051207
			// print '<div>item: </div>'; print_r($row); print '<hr>';
		}
		return $items;
	}
	
	function getDisplayByGroupWhere($where_array) {
		global $USER;
		$can_access_users = array($USER['user_id']);
		$access_db =& $this->mod->getDatabaseTable('access');
		
		// print '<div>getDisplayByGroupWhere</div>';
		// Phil 20061025 Get all users or just selected users
		$flag = false;
		if (isset($this->selected_users_only) && $this->selected_users_only) {
			// print '<div>flag set</div>';
			$flag = true;
		}
		
		$result = $access_db->getViewUsers($flag);
		// $this->db->printError();
		if (is_array($result)) {
			foreach ($result as $k => $v) {
				if ($v['access']) {
					$can_access_users[] = $v['creator_id'];
				}
			}
		}
		$can_access_users = array_unique($can_access_users);
		$can_access_users = implode(',', $can_access_users);
		// print '<div>can_access_users: ' . $can_access_users . '</div>';
		$where_array[] = 
			'
			(  
				Calendar__item.creator_id IN (' . $can_access_users  . ')' .  
			  ' OR 
				Calendar__item_user.user_id IN (' . $can_access_users . ')' . 
			' 
			) AND Calendar__item.is_cancelled=0'
		; 
		return $where_array;
	}
	 
	function processGetItemsContactFields(&$tables, &$fields) {
		$contact_fields = array();
		// Handles contact fields
		foreach ($this->getDataDefinitionFields() as $k => $v) {
			if ($v['type'] == 'contact') {
				$contact_fields[] = $v['field_key'];
			}
		}
		if (count($contact_fields)) {
			$columns = array();
			$columns[] = $this->getTableName() . '.*';
			if (!strlen($tables)) { // Phil 20060525
				$tables = $this->getTableName();
			}
			foreach ($contact_fields as $k => $v) {
				// Phil 20060525
				$tables .= ' LEFT JOIN Contact__item _' . $v .
					' ON ' . $this->getTableName() . '.' . $v . ' = _' . $v . '.id' ;
				$columns[] = '_' . $v . '.display_name' . ' AS _' . $v . '_display_name';	
			}
			$fields = implode(', ', $columns);
		} else {
			$tables = $this->getTableName();
			$fields = $this->getTableName() . '.*';
		}
		return $contact_fields;
	}
	
	function getPeriodItems($view_users, $start_time, $end_time) {
		$t1 = $this->table_name;
		$t2 = $this->module_name . '__' . $this->table_name_user;
		$t3 = $this->module_name . '__' . 'category_dropdown';
		$t4 = $this->module_name . '__' . 'status_dropdown'; // Phil 20060524
		$t5 = $this->module_name . '__' . 'type_dropdown'; // Phil 20070426
		
		$join_table_name = $this->join_tables . " LEFT JOIN $t3 ON $t1.category_id=$t3.id";
		$join_table_name .= " LEFT JOIN $t4 ON $t2.status_id = $t4.id"; // Phil 20060524
		$join_table_name .= " LEFT JOIN $t5 ON $t1.type_id = $t5.id"; // Phil 20070426
		
		$user_where = '';
		foreach ($view_users as $user_id => $val) {
			$user_where .= " $t2.user_id=" . $user_id .' or';			 			 
		}
		$user_where = '(' . substr($user_where,0,-3) . ')';
		
		$where = "(($t1.date_start>=$start_time AND $t1.date_start<=$end_time) OR (($t1.date_start+$t1.duration)>=$start_time AND ($t1.date_start+$t1.duration)<=$end_time)) AND $user_where AND $t1.is_cancelled=0";
		// Phil 20060524
		$fields = "$t1.title, $t1.content, $t3.name AS category_name, $t4.name AS status_name, $t1.date_start, $t1.duration, $t2.item_id, $t2.user_id";
		// Phil 20060525
		$fields .= ', _contact_id.id' . ' AS _contact_id_id ';	
		$fields .= ', _contact_id.display_name' . ' AS _contact_id_display_name ';	
		// Phil 20070426
		$fields .= ", $t5.name AS type_name ";	
		
		// Phil 20060525
		$tables = $join_table_name;
		
		$foreign_fields = $this->processGetItemsForeignFields($tables, $fields); // Alters the where statement for foreign fields
		$result = $this->db->quickGetAllRows($fields, $tables, $where);
		$result = $this->processContactPhoneNumbers($result);
		
		// print_r($result); print '<hr>';
		return $result;
	}
	
	/**
	 * @author Phil Calder
	 */
	function processContactPhoneNumbers($result) {
		// Phil 20060717 Adding option for multiple phone numbers
		if (is_array($result)) {
			global $SERVICES;
			$preferences =& $SERVICES->getService('Preferences');
			$show_multiple_ph_numbers = $preferences->getUserPref('calendarShowMultipleContactPhNumbers');
			if ($show_multiple_ph_numbers) {
				foreach ($result as $k => $v) {
					
					// Phil 20061019
					// If we have a custom fields, get them here:
					$custom_fields = $this->getCustomFields($v);
					if (strlen($custom_fields)) {
						$result[$k]['__other_contact_info'] = $custom_fields;
					}
					
					
					// If we have a contact and a primary phone number
					if (isset($v['_contact_id_id']) && $v['_contact_id_id'] && isset($v['_AddressPhone__item_content']) && $v['_AddressPhone__item_content']) {
						$other_phone_numbers_array = array();
						$other_phone_numbers = $this->db->quickGetAllRows(
							'label, content, primary_record', 
							'AddressPhone__item', 
							'contact_id = "' . $v['_contact_id_id'] . '" AND active=1', 
							'priority ASC'
						);
						if (is_array($other_phone_numbers)) {
							foreach ($other_phone_numbers as $ph_k => $ph_v) {
								if ($ph_v['primary_record']) {
									$result[$k]['_AddressPhone__item_content'] = (strlen($ph_v['label']) ? $ph_v['label'] . ': ' : '') . $ph_v['content'];
								} else {
									$other_phone_numbers_array[] = (strlen($ph_v['label']) ? $ph_v['label'] . ': ' : '') . $ph_v['content'];
								}
							}
							if (count($other_phone_numbers_array)) {
								$result[$k]['__other_phone_numbers'] = implode('<br>', $other_phone_numbers_array);
							}
						}
					}
					
				}
			}
		}
		return $result;
	}
	
	// Phil 20061019
	function getCustomFields($v) {
		// print '<div>TODO: ' . get_class($this) . '::getCustomFields</div>'; print_r($v); print '<hr>';
		return false;
		
		global $SERVICES;
		$config =& $SERVICES->getService('Config');
		$result = false;
		// TODO: modules_ext/Calendar/module.config
		$field_info = $config->lookup('modules.Calendar.file_set.item.custom_fields');
		// $field_info = array(
			// 'spouse' => array(
				// 'label' => 'Spouse',
				// 'field' => 'spouse'
			// )
		// );
		if (is_array($field_info)) {
			// print '<div>getCustomFields - 1</div>';
			$field_list = '';
			foreach ($field_info as $fk => $field) {
				if (is_array($field)) {
					if (isset($field['field']) && $field['field']) {
						$field_list .= $field['field'] . ',';
					}
				}
			}
			$field_list = rtrim($field_list, ',');
			$field_list = trim($field_list);
			
			
			// If we have a contact
			if (isset($v['_contact_id_id']) && $v['_contact_id_id'] && strlen($field_list)) {
				// $field_list = 'custom_id, ' . $field_list;
				// print '<div>getCustomFields - field_list: ' . $field_list . '</div>';
				$other_contact_info = array();
				$other_contact_info = $this->db->quickGet(
					$field_list,  // comma separated list of custom field names, eg 'myfield1, myfield2'
					'Contact__item_custom', 
					'custom_id = "' . $v['_contact_id_id'] . '"'
				);
				if ($this->db->hasError()) {
					// print '<div>getCustomFields - db error</div>';
					// $this->db->printError();
					return false;
				}
				if ($other_contact_info !== false && !is_array($other_contact_info)) {
					$other_contact_info = array($field_list => $other_contact_info); // Only 1 field
				}
				// print '<div>contact id: ' . $v['_contact_id_id'] . '</div>'; print_r($other_contact_info); print '<hr>'; // exit();//TODO **********************
				if (is_array($other_contact_info)) {
					// foreach ($other_contact_info as $k => $v) {
						// $other_contact_info_array[] = (strlen($v['label']) ? $v['label'] . ': ' : '') . $v['content'];
					// }
					foreach ($field_info as $fk => $field) {
						if (is_array($field)) {
							if (isset($field['field']) && $field['field']) {
								$field_list .= $field['field'] . ',';
								$other_contact_info_array[] = (strlen($field['label']) ? $field['label'] . ': ' : '') . $other_contact_info[$field['field']];
							}
						}
					}
					// print '<div>other_contact_info_array</div>'; print_r($other_contact_info_array); print '<hr>';
					if (count($other_contact_info_array)) {
						$result = implode('<br>', $other_contact_info_array);
					}
				}
			}
		}
		// print '<div>returning: ' .$result . '</div>';
		return $result;
	}
					
	 
	 function getDCalItems($current_date_str) {
		 global $SERVICES, $USER;
		 $t1 = $this->table_name;
		 $t2 = $this->module_name . '__' . $this->table_name_user;
		 $current_date_pieces = explode('-' , $current_date_str);
		 $start_time = $SERVICES->offsetTimeZoneForWrite(mktime(0, 0, 0, $current_date_pieces[1], $current_date_pieces[2],$current_date_pieces[0]));
		 $end_time = $SERVICES->offsetTimeZoneForWrite(mktime(23, 59, 59, $current_date_pieces[1], $current_date_pieces[2], $current_date_pieces[0]));
		 $fields = "$t1.id, $t1.is_cancelled";
		 $where = "(($t1.date_start>=$start_time AND $t1.date_start<=$end_time) OR (($t1.date_start+$t1.duration)>=$start_time AND ($t1.date_start+$t1.duration)<=$end_time)) AND $t2.user_id=" . $USER['id'] . " AND $t1.is_cancelled=0";		 
		 return  $this->db->quickGet($fields, $this->join_tables, $where, '', '');
	 } 
	 
	 function getYMDItems($view_users, $year, $month, $day) {
		 global $SERVICES;
		 $t1 = $this->table_name;
		 $t2 = $this->module_name . '__' . $this->table_name_user;
		 $start_time = $SERVICES->offsetTimeZoneForWrite(mktime(0, 0, 0, $month, $day, $year));
		 $end_time = $SERVICES->offsetTimeZoneForWrite(mktime(23, 59, 59, $month, $day, $year));	 
		 $user_where = '';
		 foreach ($view_users as $user_id => $val) {
			 $user_where .= " $t2.user_id=" . $user_id .' or';			 			 
		 }
		 $user_where = '(' . substr($user_where,0,-3) . ')';
		 $fields = "count( $t2.id ) as total , $t2.user_id, $t1.title"; // Phil 20070710 added title
		 $where = "(($t1.date_start>=$start_time AND $t1.date_start<=$end_time) OR (($t1.date_start+$t1.duration)>=$start_time AND ($t1.date_start+$t1.duration)<=$end_time)) AND $user_where AND $t1.is_cancelled=0";
		 $extra = "GROUP BY $t2.user_id"; 
		 $result = $this->db->select($fields, $this->join_tables, $where, '', $extra);
		 $rows = array();
		 while ($row = $this->db->getRow($result)) {
			$new_row = (sizeof(array_keys($row)) == 1 ? current($row) : $row);
			$rows[$new_row['user_id']] = $new_row;
		 }
		 return $rows;
	 }
	 
	 function getAlertIds($limit = '') {
		global $USER,$SERVICES;
		$t1 = $this->table_name;
		$t2 = $this->module_name . '__' . $this->table_name_user;
		$t3 = $t1 . '_seen'; // Phil 20051124
		
		$where = $this->getWhereStatement();
		$cur_time = time();
		// Phil 20060702 Added is_cancelled check
		$where .= " AND $t1.is_cancelled=0 AND $t2.user_id=" . $USER['id'] . " AND $t2.alert_on=1 AND ($t1.date_start - $t2.alert_in_advance)<=$cur_time"   
				.  " AND (($t1.date_start + $t2.alert_end)>=$cur_time OR $t2.alert_end=-1) ";
		
		// Phil 20051124
		$tables = $this->join_tables . " LEFT JOIN $t3 ON $t1.id = $t3.item_id AND $t3.user_id = '" . $USER['user_id'] . "'";
		$fields = "$t1.id, $t2.user_id";
		$this->db->select($fields, $tables, $where , '' , '' , '');
		
		// Phil 20060104 storing query parts in class variables, see getCountAlertsQuery
		$this->count_alerts_query = array();
		$this->count_alerts_query['fields'] = $fields;
		$this->count_alerts_query['tables'] = $tables;
		$this->count_alerts_query['where'] = $where;
		
		$result = $this->db->select(
			$this->count_alerts_query['fields'], 
			$this->count_alerts_query['tables'], 
			$this->count_alerts_query['where']
		);
		
		if ($this->db->hasError()) {
			$this->db->printError();
			return false;
		}
		
		while ($row = $this->db->getRow($result)) {
			$items[] = $row['id'];
		}
		return $items;	
	}
	 
	/**
	 * Set the data deinition array
	 *
	 * @param array $data_definition An array containing the data definition (fields and options)
	 */
	function setDataDefinition($data_definition){
		global $SERVICES;
		$SERVICES->getService('Preferences');
		$this->mod =& $SERVICES->modules->getModule('Calendar');
		if (is_array($data_definition['fields'])) { // Phil 20051107 - it should be, but better to check
			foreach($data_definition['fields'] as $k => $v){
				switch ($v['field_key']){
					case 'alert_in_advance':
					$data_definition['fields'][$k]['default_value'] = $SERVICES->preferences->getUserPref('calendarDefaultAlertAdvance');
						$data_definition['fields'][$k]['dropdown_values'] = $this->mod->getSharedVars('alert_in_advance');
					break;
					case 'alert_end':
						$data_definition['fields'][$k]['default_value'] = $SERVICES->preferences->getUserPref('calendarDefaultAlertEnd');					
						$data_definition['fields'][$k]['dropdown_values'] = $this->mod->getSharedVars('alert_end');
					break;
				}
			}
		}
		parent::setDataDefinition($data_definition);
	}
}

class item_seenCalendarDatabaseTable extends WebMATDatabaseDetailTable {
	
	/**
	 * The constructor class for item_seenCalendarDatabaseTable
	 *
	 * @param string $module_name the module name
	 * @param array $data_definition the data definition
	 */
	
	function item_seenCalendarDatabaseTable($module_name, $data_definition) {
		global $USER;
		$this->table_name = 'item_seen';
		$this->WebMATDatabaseTable($module_name, $data_definition); //calls the constructor for WebMATDatabaseDetailTable
	}

	function setItem($item) {
		global $SERVICES, $USER, $L;
		$item = $this->setPermissionsFields($item);
		//insert new record
		$this->db->insert($this->getTableName(), $item);
		$id = $this->db->getInsertId();
		
		if ($this->db->hasError()) {
			$this->db->printError();
		} else {
			$this->setLog($L['logCreate'], 'ID=' . $id);
			$this->setItemSeenStatus($id, 1);
		}
	}
	
	function getItems($order_by = false, $limit = false, $where = false) {  //get all
		global $SERVICES, $USER;
		$contact_fields = array();
		$tables = $fields = '';	// set in processGetItemsContactFields and processGetItemsForeignFields
		$contact_fields = $this->processGetItemsContactFields($tables, $fields);
		$foreign_fields = $this->processGetItemsForeignFields($tables, $fields);
		$this->processGetItemsItemUserTable($tables, $fields);
		$sort = $this->getOrderBy($order_by);
		if ($where) {
			$where = $where;
		} else {
			$where = $this->getWhereStatement();
		}
		 
		$result = $this->db->select($fields, $tables, $where, $sort, '',$limit);
		if ($this->db->hasError())	$this->db->printError();
		while ($row = $this->db->getRow($result)) {
			$items[] = $row;
		}
		return $items;
	}
}

class item_userCalendarDatabaseTable extends WebMATDatabaseDetailTable {
	
	/**
	 * @var $item_cancelled
	 * @access private
	 */
	var $item_cancelled = array();
	
	/**
	 * @var $access_cache
	 * @access private
	 */
	var $access_cache = array();
	
	/**
	 * The constructor class for itemCalendarDatabaseTable
	 *
	 * @param string $module_name the module name
	 * @param array $data_definition the data definition
	 */
	function item_userCalendarDatabaseTable($module_name, $data_definition) {
		global $USER;
		$this->table_name = 'item_user';
		$this->WebMATDatabaseTable($module_name, $data_definition); //calls the constructor for WebMATDatabaseDetailTable
	}
	
	function isEditable($id,  $item = false) {
		$id = $this->cleanId($id); // Phil 20070305
		// Phil 20061013 cached
		if (
			isset($this->cache[$this->module_name][$this->getTableName()][$id]['editable']) 
			&& $this->cache[$this->module_name][$this->getTableName()][$id]['editable']
		) {
			// print '<div>isEditable(' . $id . ') cached</div>';
			return $this->cache[$this->module_name][$this->getTableName()][$id]['editable'];
		}
		
		// print '<div>' . get_class($this) . '::isEditable(' . $id . ')</div>'; print_r($item); print '<hr>';
		global $USER;
		if (!$id) {
			return ($this->cache[$this->module_name][$this->getTableName()][$id]['editable'] = false);
		}
		if (!$item || !isset($item['role_id'])) {
			$item = $this->db->quickGet(
				// 'user_id', 
				$this->table_name . '.role_id, ' . $this->table_name . '.user_id, ' . $this->table_name . '.item_id', 
				$this->table_name . ' LEFT JOIN Calendar__item ON Calendar__item.id=' . $this->table_name . '.item_id', 
			 	$this->table_name . ".id=$id AND Calendar__item.is_cancelled=0 AND Calendar__item_user.role_id<>'" . $this->role_no_need_to_attend . "'"
			);
			// print '<div>item from db:</div>'; print_r($item); print '<hr>';
			// $this->db->printError();
			if (!$item['item_id']) {
				// print '<div>false - no db item</div>';
				return ($this->cache[$this->module_name][$this->getTableName()][$id]['editable'] = false);
			}
			// $item['role_id'] = $role_id;
		}
		
		if ($item['role_id'] == $this->role_no_need_to_attend) {
			// print '<div>false - no need to attend</div>';
			return ($this->cache[$this->module_name][$this->getTableName()][$id]['editable'] = false);
		}
		
		// Phil 20060524 Cached value returned
		if ($this->isCancelled($item['item_id'])) {
			// print '<div>false - is cancelled</div>';
			return ($this->cache[$this->module_name][$this->getTableName()][$id]['editable'] = false);
		} 
		
		if ($item['user_id'] == $USER['id']) {
			return ($this->cache[$this->module_name][$this->getTableName()][$id]['editable'] = true);
		}
		// Phil 20060524 Uses cached value
		// print '<div>test - access for ' . $USER['id'] . ' to user ' . $item['user_id'] . '</div>';
		$access = $this->getAccess($item['user_id'], $USER['id']);
		if ($access >= 2) {
			return ($this->cache[$this->module_name][$this->getTableName()][$id]['editable'] = true);
		}
		// print '<div>false - default</div>';
		return ($this->cache[$this->module_name][$this->getTableName()][$id]['editable'] = false);
	}
	
	/**
	 * Queries the Calendar item table. Results are cached.
	 * 
	 * @param int $item_id The Calendar item id
	 * @return boolen Whether the Calendar item is cancelled
	 * @author Phil Calder
	 */
	function isCancelled($item_id) {
		if ( $item_id && isset($this->item_cancelled) && is_array($this->item_cancelled) && array_key_exists($item_id, $this->item_cancelled) ) {
			return $this->item_cancelled[$item_id];
		}
		$result = $this->db->quickGet('is_cancelled', 'Calendar__item', 'id=' . $item_id);
		$this->item_cancelled[$item_id] = ($result ? true : false);
		return $this->item_cancelled[$item_id];
	}
	
	/**
	 * Queries the Calendar item table. Results are cached.
	 * 
	 * @param int $item_id The Calendar item id
	 * @return boolen Whether the Calendar item is cancelled
	 * @author Phil Calder
	 */
	function getAccess($creator_id, $user_id) {
		// TODO: item and item_user tables both have a get access method. 
		// One should call the other
		if ($creator_id && $user_id && isset($this->access_cache) && is_array($this->access_cache) ) {
			if (array_key_exists($creator_id, $this->access_cache) ) {
				if (array_key_exists($user_id, $this->access_cache[$creator_id]) ) {
					return $this->access_cache[$creator_id][$user_id];
				}
			}
		}
		
		$this->access_cache[$creator_id][$user_id] = $this->db->quickGet(
			'access', 
			'Calendar__access', 
			'creator_id=' . $creator_id . ' AND user_id=' . $user_id
		);
		return $this->access_cache[$creator_id][$user_id];
	}
	
	function getAlertIds($limit = '') {
		return false;
	}
	 
	function setUnSeenFields($item_id) {
		global $USER;
		$id = $this->db->quickGet('id', $this->table_name, "item_id=$item_id AND user_id=" . $USER['id']);
		if ($id) {
			$this->db->update($this->table_name, array('seen'=>1), "id=$id");
		}
	}
	 
	function setDataDefinition($data_definition){
		global $SERVICES;
		$SERVICES->getService('Preferences');
		$this->mod =& $SERVICES->modules->getModule('Calendar');
		if (is_array($data_definition['fields'])) { // Phil 20051107 - it should be, but better to check
			foreach($data_definition['fields'] as $k => $v){
				switch ($v['field_key']){
					case 'alert_in_advance':
					$data_definition['fields'][$k]['default_value'] = $SERVICES->preferences->getUserPref('calendarDefaultAlertAdvance');
						$data_definition['fields'][$k]['dropdown_values'] = $this->mod->getSharedVars('alert_in_advance');
					break;
					case 'alert_end':
						$data_definition['fields'][$k]['default_value'] = $SERVICES->preferences->getUserPref('calendarDefaultAlertEnd');					
						$data_definition['fields'][$k]['dropdown_values'] = $this->mod->getSharedVars('alert_end');
					break;
				}
			}
		}
		parent::setDataDefinition($data_definition);
	}
}

class category_dropdownCalendarDatabaseTable extends WebMATDatabaseDetailTable {
	
	/**
	 * The constructor class for itemCalendarDatabaseTable
	 *
	 * @param string $module_name the module name
	 * @param array $data_definition the data definition
	 */
	function category_dropdownCalendarDatabaseTable($module_name, $data_definition) {
		global $USER;
		$this->table_name = 'category_dropdown';
		$this->WebMATDatabaseTable($module_name, $data_definition); //calls the constructor for WebMATDatabaseDetailTable
	}
}

class settingsCalendarDatabaseTable extends WebMATDatabaseDetailTable {
	
	/**
	 * The constructor class for itemCalendarDatabaseTable
	 *
	 * @param string $module_name the module name
	 * @param array $data_definition the data definition
	 */
	function settingsCalendarDatabaseTable($module_name, $data_definition) {
		global $USER;
		$this->table_name = 'settings';
		$this->WebMATDatabaseTable($module_name, $data_definition); //calls the constructor for WebMATDatabaseDetailTable
	}
}

class accessCalendarDatabaseTable extends WebMATDatabaseDetailTable {
	
	/**
	 * The constructor class for itemCalendarDatabaseTable
	 *
	 * @param string $module_name the module name
	 * @param array $data_definition the data definition
	 */
	function accessCalendarDatabaseTable($module_name, $data_definition) {
		global $USER;
		$this->table_name = 'access';
		$this->WebMATDatabaseTable($module_name, $data_definition); //calls the constructor for WebMATDatabaseDetailTable
	}
	
	function getViewUsers($selected_only = false) {
		global $USER;
		$fields = 'creator_id, access, selected';
		$where = 'user_id=' . $USER['id'] . ' AND access>0 AND active=1';
		if ($selected_only) {
			$where .= ' AND selected>0';
		}
		return $this->db->quickGetAllRows(
			$fields, 
			$this->table_name, 
			$where
		);
	}
}
?>