<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Services
 */
 
/**
 * Handles user and company preferences.
 * 
 * @author Phil Calder
 * @package Services
 */
class Preferences extends Service {
	
	/**
	 * @access private
	 */
	var $auth;
	/**
	 * @access private
	 */
	var $db;
	
	/**
	 * Constructor initialises required class variables
	 */
	function Preferences() {
		global $SERVICES, $USER;
		$this->auth =& $SERVICES->getService('Authentication');
		$this->db =& $SERVICES->getService('Database');
		if (!isset($USER) || !is_array($USER)) {
			$USER = array();
		}
		if (!isset($USER['prefs']) || !is_array($USER['prefs'])) {
			$USER['prefs'] = array();
		}
		$this->user_pref =& $USER['prefs']; // Phil 20061128 Store cached values in user session
	}
	
	/**
	 * @param $key The preference key (name)
	 * @param $default_value - The optional value to set if this preference does not exist.
	 * @param $user_id The optional user id. Note: Only authorised adminstrators can obtain another users prefernce value. Will default to the current user.
	 * @return The preference value, or false.
	 */
	function getUserPref($key, $default_value = '', $user_id = 0) {
		global $USER;
		
		if (!$user_id || (!$this->auth->writeAccess($USER['user_id'], 'UserAdmin'))) {
			$user_id = (isset($USER['user_id']) ? $USER['user_id'] : 0); // this is the default (ie get pref for current user)
		}
		
		if (!isset($this->user_pref[$user_id][$key])) { // Phil 20040923
			// $this->printDebug('Preferences::getUserPref user_id: ' . $user_id . ', key: ' . $key . ' is not cached');
			$pref = false;
			
			if ($user_id) {
				$pref = $this->db->quickGet(
					'pref_value', 
					'System__prefs_user, System__prefs_key', 
					'pref_id = System__prefs_key.id AND user_id = ' . $user_id . ' AND pref_key = "' . $key . '"'
				);
			}
			
			// $this->printDebug('Preferences::getUserPref user_id: ' . $user_id . ', key: ' . $key . ' pref from database is: ' . $pref);
			if ( $pref === false ) {
				$pref = $this->getCompanyPref($key, $default_value);
			}
			
			$this->user_pref[$user_id][$key] = $pref;
		}
		
		// $this->printDebug('Preferences::getUserPref key: ' . $key . ', value: ' . $this->user_pref[$user_id][$key] . ', user_id: ' . $user_id);
		return $this->user_pref[$user_id][$key];
	}
	
	/**
	 * @param $key The preference key (name)
	 * @param $default_value - The optional value to set if this preference does not exist.
	 * @return The preference value, or false.
	 */
	function getCompanyPref($key, $default_value = '') {
		global $USER;
		$pref = $this->db->quickGet(
			'pref_value',
			'System__prefs_user, System__prefs_key',
			'pref_id = System__prefs_key.id AND user_id = 0 AND pref_key = "' . $key . '"'
		);
		
		if ( $pref === false ) {
			if ($default_value != '') {
				$pref = $default_value;
				$this->setUserPref($key, $default_value);
				
			} else {
				$pref = $this->db->quickGet(
					'default_value',
					'System__prefs_key',
					'pref_key = "' . $key . '"'
				);
			}
		}
		
		// $this->printDebug('Preferences::getCompanyPref key: ' . $key . ', default_value: ' . $default_value . ', returning ' . $pref);
		return $pref;
	} // getCompanyPref	
	
	/**
	 * @param $key The preference key (name)
	 * @param $value - The value to set for this preference.
	 * @param $user_id The optional user id. Note: Only authorised adminstrators set a preference for another user. Will default to the current user.
	 */
	function setUserPref($key, $value, $user_id = 0) {
		global $USER;
		
		// if (empty($user_id) || (!$this->auth->writeAccess($user_id, 'UserAdmin'))) {
			// $user_id = $USER['user_id'];
		// }
		// Phil 20070129 Need to check if current user can change prefs for other users
		if (empty($user_id) || (!$this->auth->writeAccess($USER['user_id'], 'UserAdmin'))) {
			$user_id = $USER['user_id'];
		}
		
		// $this->printDebug('Preferences::setUserPref key: ' . $key . ', value: ' . $value . ', user_id: ' . $user_id);
		$pref_id = $this->getPrefId($key);
		
		// $this->printDebug('Preferences::setUserPref key (id): ' . $key . ', pref_id: ' . $pref_id);
		if (empty($pref_id)) {
			// Create a new preference
			$pref_id = $this->addUserPref($key);
			// $this->printDebug('Preferences::setUserPref addUserPref returned pref_id: ' . $pref_id);
		}
		
		// $this->printDebug('Preferences::setUserPref calling removeUserPref');
		$this->removeUserPref($pref_id, $user_id);
		// if (!empty($value) || ($value !== 0)) {
		if (!empty($value) || (empty($value) && (($value === '0') || ($value === 0))) ) {
			// $this->printDebug('Preferences::setUserPref value not empty,  calling insertUserPref ' . $pref_id . ', ' . $value . ', ' . $user_id);
			$this->insertUserPref($pref_id, $value, $user_id);
		}
		// $this->printDebug('<hr>');
	}
	
	/**
	 * @param $key The preference key (name)
	 * @param $value - The optional value to set for this preference.
	 * @param $override Optional, means this company preference will override all existing users preferences for the given key.
	 */
	function setCompanyPref($key, $value, $override = 0) {
	
		if (! ($pref_id = $this->getPrefId($key))) {
			return; // pref does not exist
		}
		
		if (isset($override) && $override) {
			$this->removeUserPref($pref_id, 'All'); // remove pref for all users (incl company)
		} else {
			$this->removeUserPref($pref_id, '0'); // remove company pref
		}
		
		// if (!empty($value)) {
		$this->insertUserPref($pref_id, $value, '0'); // set the company pref
		// }
	}
	
	/**
	 * Blocks users from setting their user preference for this key.
	 *
	 * @param $items An array of preference ids to block.
	 */
	function setBlockedPref($items) {
		// Remove all blocked preferences
		$this->db->delete(
			'System__prefs_block', 
			'1'
		);
		
		// Add blocked preferences
		if (is_array($items) && sizeof($items)) {
			foreach ($items as $k => $key) {
				if ($pref_id = $this->getPrefId($key)) {
					$this->db->insert(
						'System__prefs_block', 
						array('pref_id' => $pref_id)
					);
				}
			}
		}
	}
	
	/**
	 * @param $key The preference key.
	 * @return boolean false if the given preference is 'blocked', or a 'true' (greater than zero) value.
	 */
	function isBlockedPref($key) {
		if ($pref_id = $this->getPrefId($key)) {
			return $this->db->quickGet('pref_id', 'System__prefs_block', 'pref_id = ' . $pref_id);
		}
		return false;
	}
	
	// ---- Mostly internal methods below -----
	
	/**
	 * Internal
	 * @access private
	 */
	function getPrefId($key) {
		$id = '';
		if (!empty($key)) {
			$id = $this->db->quickGet(
				'id', 
				'System__prefs_key', 
				'pref_key = "' . $key . '"'
			);
		}
		// print '<p>getPrefId(' . $key . '): ' . $id . '</p>';
		return $id;
	}
	
	/**
	 * Internal
	 * @access private
	 */
	function getPrefKey($id) {
		$key = '';
		if (!empty($id)) {
			$key = $this->db->quickGet(
				'pref_key', 
				'System__prefs_key', 
				'id = "' . $id . '"'
			);
		}
		// print '<p>getPrefKey(' . $id . '): ' . $key . '</p>';
		return $key;
	}
	
	/**
	 * Internal
	 * @access private
	 */
	function addUserPref($key, $default_value = '') {
		$id = false;
		$values = array();
		
		if (!empty($key)) {
			$values['pref_key'] = $key;
			
			if (!empty($default_value)) {
				$values['default_value'] = $default_value;
			}
			
			$id = $this->db->insert(
				'System__prefs_key',
				$values
			);
			
			if ($this->db->hasError()) {
				$this->db->printError();	
			}
		}
		
		// print '<p>addUserPref: ' . $id . '</p>';
		return $id;
	}
	
	/**
	 * Internal
	 *
	 * @access private
	 */
	function removeUserPref($pref_id, $user_id) {
		
		$user_query = "user_id = '$user_id'";
		$pref_query = "pref_id = '$pref_id'";
		
		if ($user_id == 'All') {
			$user_query = '1'; // Remove pref for all users
		}
		
		$result = $this->db->delete(
			'System__prefs_user',
			// 'user_id = ' . $user_id . ' AND pref_id = ' . $pref_id
			$user_query . ' AND ' . $pref_query
		);
	}
	
	/**
	 * Internal
	 * @access private
	 */
	function insertUserPref($pref_id, $pref_value, $user_id) {
		// $this->printDebug('Preferences::insertUserPref pref_id: ' . $pref_id . ', pref_value: ' . $pref_value . ', user_id: ' . $user_id);
		$this->db->insert(
			'System__prefs_user', 
			array(
				'user_id' => $user_id,
				'pref_id' => $pref_id, 
				'pref_value' => $pref_value
			)
		);
		
		$key = $this->getPrefKey($pref_id);
		$this->user_pref[$user_id][$key] = $pref_value;
	}
	
} // class Preferences
 
?>